import { jsx as _jsx } from "react/jsx-runtime";
import { Box, CircularProgress, Sheet } from "@mui/joy";
import { nanoid } from "@reduxjs/toolkit";
import { Suspense, lazy, useRef, useState, useMemo, useEffect, } from "react";
import { createPortal } from "react-dom";
import { useSelector } from "react-redux";
import "react-resizable/css/styles.css";
import { nerdDraggableDialogIndexSelector, popNerdDraggableDialogOrder, pushNerdDraggableDialogOrder, } from "../redux/nerdDraggableDialog";
import { useDispatch } from "react-redux";
const Draggable = lazy(() => import("react-draggable"));
const Resizable = lazy(() => import("react-resizable").then((module) => ({ default: module.Resizable })));
/**
 *
 * @param {PropsWithChildren<NerdDraggableDialogProps>} props props
 * @returns {ReactElement} NerdDraggableDialog
 */
export function NerdDraggableDialog(props) {
    const { children, sheetProps, draggableProps, initialHeight, initialWidth, resizableProps, resizable, } = props;
    const nodeRef = useRef(null);
    const [bounds, setBounds] = useState({
        left: 0,
        top: 0,
        bottom: 0,
        right: 0,
    });
    const onStart = (_event, uiData) => {
        const { clientWidth, clientHeight } = window.document.documentElement;
        const targetRect = nodeRef?.current?.getBoundingClientRect();
        if (targetRect)
            setBounds({
                left: -targetRect.left + uiData.x,
                right: clientWidth - (targetRect.right - uiData.x),
                top: -targetRect.top + uiData.y,
                bottom: clientHeight - (targetRect?.bottom - uiData.y),
            });
    };
    const [width, setWidth] = useState(initialWidth != undefined ? initialWidth : 600);
    const [height, setHeight] = useState(initialHeight != undefined ? initialHeight : 800);
    const rootRef = useRef(null);
    const uuid = useMemo(() => nanoid(), []);
    const index = useSelector((s) => nerdDraggableDialogIndexSelector(s, uuid));
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(pushNerdDraggableDialogOrder(uuid));
        return () => {
            dispatch(popNerdDraggableDialogOrder(uuid));
        };
    }, []);
    return createPortal(_jsx(Box, { sx: {
            zIndex: index !== -1 ? (1201 + index + 1).toString() : 1201,
            position: "fixed",
            top: "50%",
            left: "50%",
            transform: "translate(-50%, -50%)",
            pointerEvents: "none",
        }, ref: rootRef, children: _jsx(Suspense, { fallback: _jsx(Box, { display: "flex", justifyContent: "center", width: "100%", children: _jsx(Sheet, { sx: {
                        borderRadius: "md",
                        p: 3,
                        boxShadow: "xl",
                    }, ...sheetProps, children: _jsx(CircularProgress, {}) }) }), children: _jsx(Draggable, { ...draggableProps, cancel: `.react-resizable-handle${draggableProps?.cancel ? `, ${draggableProps?.cancel}` : ""}`, nodeRef: nodeRef, bounds: bounds, onStart: onStart, children: resizable ? (_jsx(Resizable, { ...resizableProps, width: width, height: height, onResize: (_e, data) => {
                        setWidth(data.size.width);
                        setHeight(data.size.height);
                    }, children: _jsx(Sheet, { ...sheetProps, sx: {
                            borderRadius: "md",
                            p: 3,
                            boxShadow: "lg",
                            ...sheetProps?.sx,
                            pointerEvents: "auto",
                            width,
                            height,
                        }, slotProps: {
                            ...sheetProps?.slotProps,
                            root: {
                                ...sheetProps?.slotProps?.root,
                                ref: nodeRef,
                            },
                        }, className: "nerdDraggableDialog", onFocus: () => {
                            dispatch(pushNerdDraggableDialogOrder(uuid));
                        }, tabIndex: 0, children: children }) })) : (_jsx(Sheet, { ...sheetProps, sx: {
                        borderRadius: "md",
                        p: 3,
                        boxShadow: "lg",
                        ...sheetProps?.sx,
                        pointerEvents: "auto",
                    }, slotProps: {
                        ...sheetProps?.slotProps,
                        root: {
                            ...sheetProps?.slotProps?.root,
                            ref: nodeRef,
                        },
                    }, onFocus: () => {
                        dispatch(pushNerdDraggableDialogOrder(uuid));
                    }, tabIndex: 0, children: children })) }) }) }), document.body);
}
